/***************************************************************************
 *   Copyright (C) 1998-2009 by authors (see AUTHORS.txt )                 *
 *                                                                         *
 *   This file is part of LuxRender.                                       *
 *                                                                         *
 *   Lux Renderer is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Lux Renderer is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *                                                                         *
 *   This project is based on PBRT ; see http://www.pbrt.org               *
 *   Lux Renderer website : http://www.luxrender.net                       *
 ***************************************************************************/

// paramset.cpp*
#include "paramset.h"
#include "error.h"
#include "constant.h"
#include <sstream>
#include <string>

using namespace lux;

// ParamSet Methods
ParamSet::ParamSet(const ParamSet &p2) {
	*this = p2;
}
/* Here we create a paramset with the argument list provided by the C API.
 * We have to 'guess' the parameter type according the the parameter's name
 */
ParamSet::ParamSet(int n, const char * pluginName, char* tokens[], char* params[])
{
	//TODO - jromang : implement this using a std::map or string hashing

	// NOTE - radiance - THIS NEEDS TO BE UPDATED! :)

	std::string pn(pluginName);
	
	for(int i=0;i<n;i++)
	{
		std::string s(tokens[i]);
		//float parameters
		if (s == "B")
			AddFloat(s,(float*)(params[i]));
		if (s == "C")
			AddFloat(s, (float*)(params[i]));
		if (s == "Pw")
			AddFloat(s, (float*)(params[i]),
				4 * FindOneInt("nu", i) * FindOneInt("nv", i));
		if (s == "Pz")
			AddFloat(s, (float*)(params[i]),
				FindOneInt("nu", i) * FindOneInt("nv", i));
		if (s == "a")
			AddFloat(s, (float*)(params[i]));
		if (s == "aconst")
			AddFloat(s, (float*)(params[i]));
		if (s == "alpha")
			AddFloat(s, (float*)(params[i]));
		if (s == "aperture")
			AddFloat(s, (float*)(params[i]));
		if (s == "aperture_diameter")
			AddFloat(s, (float*)(params[i]));
		if (s == "b")
			AddFloat(s, (float*)(params[i]));
		if (s == "baseflatness")
			AddFloat(s, (float*)(params[i]));
		if (s == "bconst")
			AddFloat(s, (float*)(params[i]));
		if (s == "brickdepth")
			AddFloat(s, (float*)(params[i]));
		if (s == "brickheight")
			AddFloat(s, (float*)(params[i]));
		if (s == "brickwidth")
			AddFloat(s, (float*)(params[i]));
		if (s == "bright")
			AddFloat(s, (float*)(params[i]));
		if (s == "bumpmapsampledistance")
			AddFloat(s, (float*)(params[i]));
		if (s == "burn")
			AddFloat(s, (float*)(params[i]));
		if (s == "cconst")
			AddFloat(s, (float*)(params[i]));
		if (s == "compo_override_alpha_value")
			AddFloat(s, (float*)(params[i]));
		if (s == "coneangle")
			AddFloat(s, (float*)(params[i]));
		if (s == "conedeltaangle")
			AddFloat(s, (float*)(params[i]));
		if (s == "contrast")
			AddFloat(s, (float*)(params[i]));
		if (s == "contrast_ywa")
			AddFloat(s, (float*)(params[i]));
		if (s == "cropwindow")
			AddFloat(s, (float*)(params[i]), 4);
/*		if (s == "data")
			AddFloat(s, (float*)(params[i]));*/ //FIXME - there's currently no way of getting the array length for regular and irregular spectrum data
		if (s == "dconst")
			AddFloat(s, (float*)(params[i]));
		if (s == "density") AddFloat(s, (float*)(params[i]), FindOneInt("nx", i) * FindOneInt("ny", i) * FindOneInt("ny", i));
		if (s == "diffusereflectreject_threshold")
			AddFloat(s, (float*)(params[i]));
		if (s == "diffuserefractreject_threshold")
			AddFloat(s, (float*)(params[i]));
		if (s == "distamount")
			AddFloat(s, (float*)(params[i]));
		if (s == "distancethreshold")
			AddFloat(s, (float*)(params[i]));
		if (s == "dmoffset")
			AddFloat(s, (float*)(params[i]));
		if (s == "dmscale")
			AddFloat(s, (float*)(params[i]));
		if (s == "econst")
			AddFloat(s, (float*)(params[i]));
		if (s == "efficacy")
			AddFloat(s, (float*)(params[i]));
		if (s == "emptybonus")
			AddFloat(s, (float*)(params[i]));
		if (s == "end")
			AddFloat(s, (float*)(params[i]));
		if (s == "energy")
			AddFloat(s, (float*)(params[i]));
		if (s == "exposure")
			AddFloat(s, (float*)(params[i]));
		if (s == "eyerrthreshold")
			AddFloat(s, (float*)(params[i]));
		if (s == "filmdiag")
			AddFloat(s, (float*)(params[i]));
		if (s == "filmdistance")
			AddFloat(s, (float*)(params[i]));
		if (s == "focaldistance")
			AddFloat(s, (float*)(params[i]));
		if (s == "fov")
			AddFloat(s, (float*)(params[i]));
		if (s == "frameaspectratio")
			AddFloat(s, (float*)(params[i]));
		if (s == "freq")
			AddFloat(s, (float*)(params[i]));
		if (s == "fstop")
			AddFloat(s, (float*)(params[i]));
		if (s == "fullsweepthreshold")
			AddFloat(s, (float*)(params[i]));
		if (s == "g")
			AddFloat(s, (float*)(params[i]));
		if (s == "gain")
			AddFloat(s, (float*)(params[i]));
		if (s == "gamma")
			AddFloat(s, (float*)(params[i]));
		if (s == "gatherangle")
			AddFloat(s, (float*)(params[i]));
		if (s == "glossyreflectreject_threshold")
			AddFloat(s, (float*)(params[i]));
		if (s == "glossyrefractreject_threshold")
			AddFloat(s, (float*)(params[i]));
		if (s == "h")
			AddFloat(s, (float*)(params[i]));
		if (s == "height")
			AddFloat(s, (float*)(params[i]));
		if (s == "hither")
			AddFloat(s, (float*)(params[i]));
		if (s == "innerradius")
			AddFloat(s, (float*)(params[i]));
		if (s == "lacu")
			AddFloat(s, (float*)(params[i]));
		if (s == "largemutationprob")
			AddFloat(s, (float*)(params[i]));
		if (s == "lensradius")
			AddFloat(s, (float*)(params[i]));
		if (s == "lightrrthreshold")
			AddFloat(s, (float*)(params[i]));
		if (s == "linear_exposure")
			AddFloat(s, (float*)(params[i]));
		if (s == "linear_fstop")
			AddFloat(s, (float*)(params[i]));
		if (s == "linear_gamma")
			AddFloat(s, (float*)(params[i]));
		if (s == "linear_sensitivity")
			AddFloat(s, (float*)(params[i]));
		if (s == "majorradius")
			AddFloat(s, (float*)(params[i]));
		if (s == "maxY")
			AddFloat(s, (float*)(params[i]));
		if (s == "maxanisotropy")
			AddFloat(s, (float*)(params[i]));
		if (s == "maxphotondist")
			AddFloat(s, (float*)(params[i]));
		if (s == "micromutationprob")
			AddFloat(s, (float*)(params[i]));
		if (s == "mindist")
			AddFloat(s, (float*)(params[i]));
		if (s == "minkovsky_exp")
			AddFloat(s, (float*)(params[i]));
		if (s == "minorradius")
			AddFloat(s, (float*)(params[i]));
		if (s == "mortarsize")
			AddFloat(s, (float*)(params[i]));
		if (s == "mutationrange")
			AddFloat(s, (float*)(params[i]));
		if (s == "nabla")
			AddFloat(s, (float*)(params[i]));
		if (s == "noiseoffset")
			AddFloat(s, (float*)(params[i]));
		if (s == "noisescale")
			AddFloat(s, (float*)(params[i]));
		if (s == "noisesize")
			AddFloat(s, (float*)(params[i]));
		if (s == "octs")
			AddFloat(s, (float*)(params[i]));
		if (s == "offset")
			AddFloat(s, (float*)(params[i]));
		if (s == "omega")
			AddFloat(s, (float*)(params[i]));
		if (s == "outscale")
			AddFloat(s, (float*)(params[i]));
		if (s == "phimax")
			AddFloat(s, (float*)(params[i]));
		if (s == "postscale")
			AddFloat(s, (float*)(params[i]));
		if (s == "power" && pn == "area")
			AddFloat(s, (float*)(params[i]));
		if (s == "prescale")
			AddFloat(s, (float*)(params[i]));
		if (s == "radius")
			AddFloat(s, (float*)(params[i]));
		if (s == "reinhard_burn")
			AddFloat(s, (float*)(params[i]));
		if (s == "reinhard_postscale")
			AddFloat(s, (float*)(params[i]));
		if (s == "reinhard_prescale")
			AddFloat(s, (float*)(params[i]));
		if (s == "relsize")
			AddFloat(s, (float*)(params[i]));
		if (s == "roughness")
			AddFloat(s, (float*)(params[i]));
		if (s == "rrcontinueprob")
			AddFloat(s, (float*)(params[i]));
		if (s == "scale")
			AddFloat(s, (float*)(params[i]));
		if (s == "screenwindow")
			AddFloat(s, (float*)(params[i]), 4);
		if (s == "sensitivity")
			AddFloat(s, (float*)(params[i]));
		if (s == "sharpness")
			AddFloat(s, (float*)(params[i]));
		if (s == "shutterclose")
			AddFloat(s, (float*)(params[i]));
		if (s == "shutteropen")
			AddFloat(s, (float*)(params[i]));
		if (s == "spheresize")
			AddFloat(s, (float*)(params[i]));
		if (s == "st")
			AddFloat(s, (float*)(params[i]),
				2 * FindOneInt("nvertices", i));  // jromang - p.926 find 'n' ? - [a ajouter dans le vecteur lors de l'appel de la fonction, avec un parametre 'nvertices supplementaire dans l API]
		if (s == "start")
			AddFloat(s, (float*)(params[i]));
		if (s == "stepsize")
			AddFloat(s, (float*)(params[i]));
		if (s == "tau")
			AddFloat(s, (float*)(params[i]));
		if (s == "temperature")
			AddFloat(s, (float*)(params[i]));
		if (s == "thetamax")
			AddFloat(s, (float*)(params[i]));
		if (s == "thetamin")
			AddFloat(s, (float*)(params[i]));
		if (s == "turbidity")
			AddFloat(s, (float*)(params[i]));
		if (s == "turbulance")
			AddFloat(s, (float*)(params[i]));
		if (s == "turbulence")
			AddFloat(s, (float*)(params[i]));
		if (s == "u0")
			AddFloat(s, (float*)(params[i]));
		if (s == "u1")
			AddFloat(s, (float*)(params[i]));
		if (s == "udelta")
			AddFloat(s, (float*)(params[i]));
		if (s == "uknots")
			AddFloat(s, (float*)(params[i]),
				FindOneInt("nu", i) + FindOneInt("uorder", i));
		if (s == "uscale")
			AddFloat(s, (float*)(params[i]));
		if (s == "uv")
			AddFloat(s, (float*)(params[i]),
				2 * FindOneInt("nvertices", i));  // jromang - p.926 find 'n' ? - [a ajouter dans le vecteur lors de l'appel de la fonction, avec un parametre 'nvertices supplementaire dans l API]
		if (s == "v0")
			AddFloat(s, (float*)(params[i]));
		if (s == "v00")
			AddFloat(s, (float*)(params[i]));
		if (s == "v01")
			AddFloat(s, (float*)(params[i]));
		if (s == "v1")
			AddFloat(s, (float*)(params[i]));
		if (s == "v10")
			AddFloat(s, (float*)(params[i]));
		if (s == "v11")
			AddFloat(s, (float*)(params[i]));
		if (s == "value")
			AddFloat(s, (float*)(params[i]));
		if (s == "variability")
			AddFloat(s, (float*)(params[i]));
		if (s == "variation")
			AddFloat(s, (float*)(params[i]));
		if (s == "vdelta")
			AddFloat(s,(float*)(params[i]));
		if (s == "vknots")
			AddFloat(s, (float*)(params[i]),
				FindOneInt("nv", i) + FindOneInt("vorder", i));
		if (s == "vscale")
			AddFloat(s, (float*)(params[i]));
		if (s == "w1")
			AddFloat(s, (float*)(params[i]));
		if (s == "w2")
			AddFloat(s, (float*)(params[i]));
		if (s == "w3")
			AddFloat(s, (float*)(params[i]));
		if (s == "w4")
			AddFloat(s, (float*)(params[i]));
		if (s == "wavelength")
			AddFloat(s, (float*)(params[i]));
/*		if (s == "wavelengths")
			AddFloat(s, (float*)(params[i]));*/ //FIXME - there's currently no way of getting the array length for irregular spectrum data
		if (s == "width")
			AddFloat(s, (float*)(params[i]));
		if (s == "xwidth")
			AddFloat(s, (float*)(params[i]));
		if (s == "yon")
			AddFloat(s, (float*)(params[i]));
		if (s == "ywa")
			AddFloat(s, (float*)(params[i]));
		if (s == "ywidth")
			AddFloat(s, (float*)(params[i]));
		if (s == "zmax")
			AddFloat(s, (float*)(params[i]));
		if (s == "zmin")
			AddFloat(s, (float*)(params[i]));

		//int parameters
		if (s == "blades")
			AddInt(s, (int*)(params[i]));
		if (s == "causticphotons")
			AddInt(s, (int*)(params[i]));
		if (s == "chainlength")
			AddInt(s, (int*)(params[i]));
		if (s == "coltype")
			AddInt(s, (int*)(params[i]));
		if (s == "costsamples")
			AddInt(s, (int*)(params[i]));
		if (s == "diffusereflectdepth")
			AddInt(s, (int*)(params[i]));
		if (s == "diffusereflectsamples")
			AddInt(s, (int*)(params[i]));
		if (s == "diffuserefractdepth")
			AddInt(s, (int*)(params[i]));
		if (s == "diffuserefractsamples")
			AddInt(s, (int*)(params[i]));
		if (s == "dimension")
			AddInt(s, (int*)(params[i]));
		if (s == "directphotons")
			AddInt(s, (int*)(params[i]));
		if (s == "directsamples")
			AddInt(s, (int*)(params[i]));
		if (s == "discardmipmaps")
			AddInt(s, (int*)(params[i]));
		if (s == "displayinterval")
			AddInt(s, (int*)(params[i]));
		if (s == "eyedepth")
			AddInt(s, (int*)(params[i]));
		if (s == "finalgathersamples")
			AddInt(s, (int*)(params[i]));
		if (s == "glossyreflectdepth")
			AddInt(s, (int*)(params[i]));
		if (s == "glossyreflectsamples")
			AddInt(s, (int*)(params[i]));
		if (s == "glossyrefractdepth")
			AddInt(s, (int*)(params[i]));
		if (s == "glossyrefractsamples")
			AddInt(s, (int*)(params[i]));
		if (s == "haltspp")
			AddInt(s, (int*)(params[i]));
		if( s == "indices")
			AddInt(s, (int*)(params[i]),
				FindOneInt("nvertices", i));  // jromang - p.926 find 'n' ? - [a ajouter dans le vecteur lors de l'appel de la fonction, avec un parametre 'nvertices supplementaire dans l API]
		if (s == "indirectphotons")
			AddInt(s, (int*)(params[i]));
		if (s == "indirectsamples")
			AddInt(s, (int*)(params[i]));
		if (s == "intersectcost")
			AddInt(s, (int*)(params[i]));
		if (s == "lightdepth")
			AddInt(s, (int*)(params[i]));
		if (s == "maxconsecrejects")
			AddInt(s, (int*)(params[i]));
		if (s == "maxdepth")
			AddInt(s, (int*)(params[i]));
		if (s == "maxphotondepth")
			AddInt(s, (int*)(params[i]));
		if (s == "maxprims")
			AddInt(s, (int*)(params[i]));
		if (s == "maxprimsperleaf")
			AddInt(s, (int*)(params[i]));
		if (s == "nlevels")
			AddInt(s, (int*)(params[i]));
		if (s == "nlights")
			AddInt(s, (int*)(params[i]));
		if (s == "noisedepth")
			AddInt(s, (int*)(params[i]));
		if (s == "nphotonused")
			AddInt(s, (int*)(params[i]));
		if (s == "nsamples")
			AddInt(s, (int*)(params[i]));
		if (s == "nsets")
			AddInt(s, (int*)(params[i]));
		if (s == "nsubdivlevels")
			AddInt(s, (int*)(params[i]));
		if (s == "nu")
			AddInt(s, (int*)(params[i]));
		if (s == "nv")
			AddInt(s, (int*)(params[i]));
		if (s == "nx")
			AddInt(s, (int*)(params[i]));
		if (s == "ny")
			AddInt(s, (int*)(params[i]));
		if (s == "nz")
			AddInt(s, (int*)(params[i]));
		if (s == "octaves")
			AddInt(s, (int*)(params[i]));
		if (s == "pixelsamples")
			AddInt(s, (int*)(params[i]));
		if (s == "power" && pn == "perspective")
			AddInt(s, (int*)(params[i]));
/*		if (s == "quadindices")
			AddInt(s, (int*)(params[i]));*/ //FIXME - there's no way of knowinf the number of elements
		if (s == "radiancephotons")
			AddInt(s, (int*)(params[i]));
		if (s == "reject_warmup")
			AddInt(s, (int*)(params[i]));
		if (s == "skipfactor")
			AddInt(s, (int*)(params[i]));
		if (s == "specularreflectdepth")
			AddInt(s, (int*)(params[i]));
		if (s == "specularrefractdepth")
			AddInt(s, (int*)(params[i]));
		if (s == "spheres")
			AddInt(s, (int*)(params[i]));
		if (s == "traversalcost")
			AddInt(s, (int*)(params[i]));
		if (s == "treetype")
			AddInt(s, (int*)(params[i]));
/*		if( s == "triindices")
			AddInt(s, (int*)(params[i]));*/ //FIXME - there's no way of knowing the number of elements
		if (s == "uorder")
			AddInt(s, (int*)(params[i]));
		if (s == "vorder")
			AddInt(s, (int*)(params[i]));
		if (s == "writeinterval")
			AddInt(s, (int*)(params[i]));
		if (s == "xresolution")
			AddInt(s, (int*)(params[i]));
		if (s == "xsamples")
			AddInt(s, (int*)(params[i]));
		if (s == "yresolution")
			AddInt(s, (int*)(params[i]));
		if (s == "ysamples")
			AddInt(s, (int*)(params[i]));

		//bool parameters
		if (s == "architectural")
			AddBool(s, (bool*)(params[i]));
		if (s == "autofocus")
			AddBool(s, (bool*)(params[i]));
		if (s == "compo_override_alpha")
			AddBool(s, (bool*)(params[i]));
		if (s == "compo_use_key")
			AddBool(s, (bool*)(params[i]));
		if (s == "compo_visible_emission")
			AddBool(s, (bool*)(params[i]));
		if (s == "compo_visible_indirect_emission")
			AddBool(s, (bool*)(params[i]));
		if (s == "compo_visible_indirect_material")
			AddBool(s, (bool*)(params[i]));
		if (s == "compo_visible_material")
			AddBool(s, (bool*)(params[i]));
		if (s == "dbg_enabledirect")
			AddBool(s, (bool*)(params[i]));
		if (s == "dbg_enableindircaustic")
			AddBool(s, (bool*)(params[i]));
		if (s == "dbg_enableindirdiffuse")
			AddBool(s, (bool*)(params[i]));
		if (s == "dbg_enableindirspecular")
			AddBool(s, (bool*)(params[i]));
		if (s == "dbg_enableradiancemap")
			AddBool(s, (bool*)(params[i]));
		if (s == "debug")
			AddBool(s, (bool*)(params[i]));
		if (s == "diffusereflectreject")
			AddBool(s, (bool*)(params[i]));
		if (s == "diffuserefractreject")
			AddBool(s, (bool*)(params[i]));
		if (s == "directdiffuse")
			AddBool(s, (bool*)(params[i]));
		if (s == "directglossy")
			AddBool(s, (bool*)(params[i]));
		if (s == "directsampleall")
			AddBool(s, (bool*)(params[i]));
		if (s == "dmnormalsmooth")
			AddBool(s, (bool*)(params[i]));
		if (s == "dmsharpboundary")
			AddBool(s, (bool*)(params[i]));
		if (s == "finalgather")
			AddBool(s, (bool*)(params[i]));
		if (s == "flipxy")
			AddBool(s, (bool*)(params[i]));
		if (s == "flipz")
			AddBool(s, (bool*)(params[i]));
		if (s == "glossyreflectreject")
			AddBool(s, (bool*)(params[i]));
		if (s == "glossyrefractreject")
			AddBool(s, (bool*)(params[i]));
		if (s == "includeenvironment")
			AddBool(s, (bool*)(params[i]));
		if (s == "indirectdiffuse")
			AddBool(s, (bool*)(params[i]));
		if (s == "indirectglossy")
			AddBool(s, (bool*)(params[i]));
		if (s == "indirectsampleall")
			AddBool(s, (bool*)(params[i]));
		if (s == "premultiplyalpha")
			AddBool(s, (bool*)(params[i]));
		if (s == "refineimmediately")
			AddBool(s, (bool*)(params[i]));
		if (s == "restart_resume_flm")
			AddBool(s, (bool*)(params[i]));
		if (s == "smooth")
			AddBool(s, (bool*)(params[i]));
		if (s == "usevariance")
			AddBool(s, (bool*)(params[i]));
		if (s == "write_exr")
			AddBool(s, (bool*)(params[i]));
		if (s == "write_exr_ZBuf")
			AddBool(s, (bool*)(params[i]));
		if (s == "write_exr_applyimaging")
			AddBool(s, (bool*)(params[i]));
		if (s == "write_exr_gamutclamp")
			AddBool(s, (bool*)(params[i]));
		if (s == "write_exr_halftype")
			AddBool(s, (bool*)(params[i]));
		if (s == "write_png")
			AddBool(s, (bool*)(params[i]));
		if (s == "write_png_16bit")
			AddBool(s, (bool*)(params[i]));
		if (s == "write_png_ZBuf")
			AddBool(s, (bool*)(params[i]));
		if (s == "write_png_gamutclamp")
			AddBool(s, (bool*)(params[i]));
		if (s == "write_resume_flm")
			AddBool(s, (bool*)(params[i]));
		if (s == "write_tga")
			AddBool(s, (bool*)(params[i]));
		if (s == "write_tga_ZBuf")
			AddBool(s, (bool*)(params[i]));
		if (s == "write_tga_gamutclamp")
			AddBool(s, (bool*)(params[i]));
		
		//string parameters
		if (s == "aamode")
			AddString(s, new string(params[i]));
		if (s == "acceltype")
			AddString(s, new string(params[i]));
		if (s == "basesampler")
			AddString(s, new string(params[i]));
		if (s == "displacementmap")
			AddString(s, new string(params[i]));
		if (s == "distmetric")
			AddString(s, new string(params[i]));
		if (s == "distribution")
			AddString(s, new string(params[i]));
		if (s == "endtransform")
			AddString(s, new string(params[i]));
		if (s == "filename")
			AddString(s, new string(params[i]));
		if (s == "filtertype")
			AddString(s, new string(params[i]));
		if (s == "iesname")
			AddString(s, new string(params[i]));
		if (s == "ldr_clamp_method")
			AddString(s, new string(params[i]));
		if (s == "mapname")
			AddString(s, new string(params[i]));
		if (s == "mapping")
			AddString(s, new string(params[i]));
		if (s == "name")
			AddString(s, new string(params[i]));
		if (s == "namedmaterial1")
			AddString(s, new string(params[i]));
		if (s == "namedmaterial2")
			AddString(s, new string(params[i]));
		if (s == "noisebasis")
			AddString(s, new string(params[i]));
		if (s == "noisebasis2")
			AddString(s, new string(params[i]));
		if (s == "noisetype")
			AddString(s, new string(params[i]));
		if (s == "photonmapsfile")
			AddString(s, new string(params[i]));
		if (s == "pixelsampler")
			AddString(s, new string(params[i]));
		if (s == "quadtype")
			AddString(s, new string(params[i]));
		if (s == "renderingmode")
			AddString(s, new string(params[i]));
		if (s == "rrstrategy")
			AddString(s, new string(params[i]));
		if (s == "scheme")
			AddString(s, new string(params[i]));
		if (s == "shutterdistribution")
			AddString(s, new string(params[i]));
		if (s == "specfile")
			AddString(s, new string(params[i]));
		if (s == "strategy")
			AddString(s, new string(params[i]));
		if (s == "subdivscheme")
			AddString(s, new string(params[i]));
		if (s == "tonemapkernel")
			AddString(s, new string(params[i]));
		if (s == "tritype")
			AddString(s, new string(params[i]));
		if (s == "type")
			AddString(s, new string(params[i]));
		if (s == "wrap")
			AddString(s, new string(params[i]));
		if (s == "write_exr_channels")
			AddString(s, new string(params[i]));
		if (s == "write_exr_compressiontype")
			AddString(s, new string(params[i]));
		if (s == "write_exr_zbuf_normalization")
			AddString(s, new string(params[i]));
		if (s == "write_png_channels")
			AddString(s, new string(params[i]));
		if (s == "write_pxr_zbuf_normalization")
			AddString(s, new string(params[i]));
		if (s == "write_tga_channels")
			AddString(s, new string(params[i]));
		if (s == "write_tga_zbuf_normalization")
			AddString(s, new string(params[i]));

		//point parameters
		if (s == "P") {
			if (pn == "nurbs")
				AddPoint(s, (Point*)(params[i]),
					FindOneInt("nu", i) * FindOneInt("nv", i));
			else
				AddPoint(s, (Point*)(params[i]),
					FindOneInt("nvertices", i)); // jromang - p.926 find 'n' ? - [a ajouter dans le vecteur lors de l'appel de la fonction, avec un parametre 'nvertices supplementaire dans l API]
		}
		if (s == "from")
			AddPoint(s, (Point*)(params[i]));
		if (s == "p0")
			AddPoint(s, (Point*)(params[i]));
		if (s == "p1")
			AddPoint(s, (Point*)(params[i]));
		if (s == "p2")
			AddPoint(s, (Point*)(params[i]));
		if (s == "to")
			AddPoint(s,(Point*)(params[i]));

		//normal parameters
		if (s == "N")
			AddNormal(s, (Normal*)(params[i]),
				FindOneInt("nvertices", i)); // jromang - p.926 find 'n' ? - [a ajouter dans le vecteur lors de l'appel de la fonction, avec un parametre 'nvertices supplementaire dans l API]
		
		//vector parameters
		if (s == "rotate")
			AddVector(s, (Vector*)(params[i]));
		if (s == "scale")
			AddVector(s, (Vector*)(params[i]));
		if (s == "sundir")
			AddVector(s, (Vector*)(params[i]));
		if (s == "translate")
			AddVector(s, (Vector*)(params[i]));
		if (s == "updir")
			AddVector(s, (Vector*)(params[i]));
		if (s == "v1")
			AddVector(s, (Vector*)(params[i]));
		if (s == "v2")
			AddVector(s, (Vector*)(params[i]));
		
		//texture parameters
		if (s == "Ka")
			AddTexture(s, std::string(params[i]));
		if (s == "Kd")
			AddTexture(s, std::string(params[i]));
		if (s == "Kr")
			AddTexture(s, std::string(params[i]));
		if (s == "Ks")
			AddTexture(s, std::string(params[i]));
		if (s == "Ks1")
			AddTexture(s, std::string(params[i]));
		if (s == "Ks2")
			AddTexture(s, std::string(params[i]));
		if (s == "Ks3")
			AddTexture(s, std::string(params[i]));
		if (s == "Kt")
			AddTexture(s, std::string(params[i]));
		if (s == "L")
			AddTexture(s, std::string(params[i]));
		if (s == "M1")
			AddTexture(s, std::string(params[i]));
		if (s == "M2")
			AddTexture(s, std::string(params[i]));
		if (s == "M3")
			AddTexture(s, std::string(params[i]));
		if (s == "R1")
			AddTexture(s, std::string(params[i]));
		if (s == "R2")
			AddTexture(s, std::string(params[i]));
		if (s == "R3")
			AddTexture(s, std::string(params[i]));
		if (s == "amount")
			AddTexture(s, std::string(params[i]));
		if (s == "bricktex")
			AddTexture(s, std::string(params[i]));
		if (s == "bumpmap")
			AddTexture(s, std::string(params[i]));
		if (s == "cauchyb")
			AddTexture(s, std::string(params[i]));
		if (s == "d")
			AddTexture(s, std::string(params[i]));
		if (s == "film")
			AddTexture(s, std::string(params[i]));
		if (s == "filmindex")
			AddTexture(s, std::string(params[i]));
		if (s == "index")
			AddTexture(s, std::string(params[i]));
		if (s == "inside")
			AddTexture(s, std::string(params[i]));
		if (s == "mortartex")
			AddTexture(s, std::string(params[i]));
		if (s == "outside")
			AddTexture(s, std::string(params[i]));
		if (s == "sigma")
			AddTexture(s, std::string(params[i]));
		if (s == "tex1")
			AddTexture(s, std::string(params[i]));
		if (s == "tex2")
			AddTexture(s, std::string(params[i]));
		if (s == "uroughness")
			AddTexture(s, std::string(params[i]));
		if (s == "vroughness")
			AddTexture(s, std::string(params[i]));
		
		//color (RGBColor) parameters
		if (s == "L")
			AddRGBColor(s, new RGBColor((float*)params[i]));
		if (s == "Le")
			AddRGBColor(s, new RGBColor((float*)params[i]));
		if (s == "compo_key_color")
			AddRGBColor(s, new RGBColor((float*)params[i]));
		if (s == "sigma_a")
			AddRGBColor(s, new RGBColor((float*)params[i]));
		if (s == "sigma_s")
			AddRGBColor(s, new RGBColor((float*)params[i]));
		if (s == "v00")
			AddRGBColor(s, new RGBColor((float*)params[i]));
		if (s == "v01")
			AddRGBColor(s, new RGBColor((float*)params[i]));
		if (s == "v10")
			AddRGBColor(s, new RGBColor((float*)params[i]));
		if (s == "v11")
			AddRGBColor(s, new RGBColor((float*)params[i]));
		if (s == "value")
			AddRGBColor(s, new RGBColor((float*)params[i]));
		
		//unknown parameter
		/*
		else
		{
			std::stringstream ss;
			ss<<"Unknown parameter '"<<p<<":"<<s<<"', Ignoring.";
			luxError(LUX_SYNTAX,LUX_ERROR,ss.str().c_str());
		}*/
	}
}

ParamSet &ParamSet::operator=(const ParamSet &p2) {
	if (&p2 != this) {
		Clear();
		u_int i;
		for (i = 0; i < p2.ints.size(); ++i)
			ints.push_back(p2.ints[i]->Clone());
		for (i = 0; i < p2.bools.size(); ++i)
			bools.push_back(p2.bools[i]->Clone());
		for (i = 0; i < p2.floats.size(); ++i)
			floats.push_back(p2.floats[i]->Clone());
		for (i = 0; i < p2.points.size(); ++i)
			points.push_back(p2.points[i]->Clone());
		for (i = 0; i < p2.vectors.size(); ++i)
			vectors.push_back(p2.vectors[i]->Clone());
		for (i = 0; i < p2.normals.size(); ++i)
			normals.push_back(p2.normals[i]->Clone());
		for (i = 0; i < p2.spectra.size(); ++i)
			spectra.push_back(p2.spectra[i]->Clone());
		for (i = 0; i < p2.strings.size(); ++i)
			strings.push_back(p2.strings[i]->Clone());
		for (i = 0; i < p2.textures.size(); ++i)
			textures.push_back(p2.textures[i]->Clone());
	}
	return *this;
}

void ParamSet::Add(ParamSet &params) {
#define ADD_PARAMSETITEM_VECTOR(VEC,ADD_METHOD) \
	for( u_int i = 0; i < VEC.size(); i++ ) { \
		ADD_METHOD(VEC[i]->name, VEC[i]->data, VEC[i]->nItems ); \
	}

	ADD_PARAMSETITEM_VECTOR(params.ints, AddInt);
	ADD_PARAMSETITEM_VECTOR(params.bools, AddBool);
	ADD_PARAMSETITEM_VECTOR(params.floats, AddFloat);
	ADD_PARAMSETITEM_VECTOR(params.points, AddPoint);
	ADD_PARAMSETITEM_VECTOR(params.vectors, AddVector);
	ADD_PARAMSETITEM_VECTOR(params.normals, AddNormal);
	ADD_PARAMSETITEM_VECTOR(params.spectra, AddRGBColor);
	ADD_PARAMSETITEM_VECTOR(params.strings, AddString);
	for( u_int i = 0; i < params.textures.size(); i++ ) {
		AddTexture(params.textures[i]->name, *(params.textures[i]->data));
	}

#undef ADD_PARAMSETITEM_VECTOR
}

void ParamSet::AddFloat(const string &name,
			            const float *data,
						int nItems) {
	EraseFloat(name);
	floats.push_back(new ParamSetItem<float>(name,
											 data,
											 nItems));
}
void ParamSet::AddInt(const string &name, const int *data, int nItems) {
	EraseInt(name);
	ADD_PARAM_TYPE(int, ints);
}
void ParamSet::AddBool(const string &name, const bool *data, int nItems) {
	EraseBool(name);
	ADD_PARAM_TYPE(bool, bools);
}
void ParamSet::AddPoint(const string &name, const Point *data, int nItems) {
	ErasePoint(name);
	ADD_PARAM_TYPE(Point, points);
}
void ParamSet::AddVector(const string &name, const Vector *data, int nItems) {
	EraseVector(name);
	ADD_PARAM_TYPE(Vector, vectors);
}
void ParamSet::AddNormal(const string &name, const Normal *data, int nItems) {
	EraseNormal(name);
	ADD_PARAM_TYPE(Normal, normals);
}
void ParamSet::AddRGBColor(const string &name, const RGBColor *data, int nItems) {
	EraseRGBColor(name);
	ADD_PARAM_TYPE(RGBColor, spectra);
}
void ParamSet::AddString(const string &name, const string *data, int nItems) {
	EraseString(name);
	ADD_PARAM_TYPE(string, strings);
}
void ParamSet::AddTexture(const string &name, const string &value) {
	EraseTexture(name);
	textures.push_back(new ParamSetItem<string>(name, (string *)&value, 1));
}
bool ParamSet::EraseInt(const string &n) {
	for (u_int i = 0; i < ints.size(); ++i)
		if (ints[i]->name == n) {
			delete ints[i];
			ints.erase(ints.begin() + i);
			return true;
		}
	return false;
}
bool ParamSet::EraseBool(const string &n) {
	for (u_int i = 0; i < bools.size(); ++i)
		if (bools[i]->name == n) {
			delete bools[i];
			bools.erase(bools.begin() + i);
			return true;
		}
	return false;
}
bool ParamSet::EraseFloat(const string &n) {
	for (u_int i = 0; i < floats.size(); ++i)
		if (floats[i]->name == n) {
			delete floats[i];
			floats.erase(floats.begin() + i);
			return true;
		}
	return false;
}
bool ParamSet::ErasePoint(const string &n) {
	for (u_int i = 0; i < points.size(); ++i)
		if (points[i]->name == n) {
			delete points[i];
			points.erase(points.begin() + i);
			return true;
		}
	return false;
}
bool ParamSet::EraseVector(const string &n) {
	for (u_int i = 0; i < vectors.size(); ++i)
		if (vectors[i]->name == n) {
			delete vectors[i];
			vectors.erase(vectors.begin() + i);
			return true;
		}
	return false;
}
bool ParamSet::EraseNormal(const string &n) {
	for (u_int i = 0; i < normals.size(); ++i)
		if (normals[i]->name == n) {
			delete normals[i];
			normals.erase(normals.begin() + i);
			return true;
		}
	return false;
}
bool ParamSet::EraseRGBColor(const string &n) {
	for (u_int i = 0; i < spectra.size(); ++i)
		if (spectra[i]->name == n) {
			delete spectra[i];
			spectra.erase(spectra.begin() + i);
			return true;
		}
	return false;
}
bool ParamSet::EraseString(const string &n) {
	for (u_int i = 0; i < strings.size(); ++i)
		if (strings[i]->name == n) {
			delete strings[i];
			strings.erase(strings.begin() + i);
			return true;
		}
	return false;
}
bool ParamSet::EraseTexture(const string &n) {
	for (u_int i = 0; i < textures.size(); ++i)
		if (textures[i]->name == n) {
			delete textures[i];
			textures.erase(textures.begin() + i);
			return true;
		}
	return false;
}
float ParamSet::FindOneFloat(const string &name,
                             float d) const {
	for (u_int i = 0; i < floats.size(); ++i)
		if (floats[i]->name == name &&
			floats[i]->nItems == 1) {
			floats[i]->lookedUp = true;
			return *(floats[i]->data);
		}
	return d;
}
const float *ParamSet::FindFloat(const string &name,
		int *nItems) const {
	for (u_int i = 0; i < floats.size(); ++i)
		if (floats[i]->name == name) {
			*nItems = floats[i]->nItems;
			floats[i]->lookedUp = true;
			return floats[i]->data;
		}
	return NULL;
}
const int *ParamSet::FindInt(const string &name, int *nItems) const {
	LOOKUP_PTR(ints);
}
const bool *ParamSet::FindBool(const string &name, int *nItems) const {
	LOOKUP_PTR(bools);
}
int ParamSet::FindOneInt(const string &name, int d) const {
	LOOKUP_ONE(ints);
}
bool ParamSet::FindOneBool(const string &name, bool d) const {
	LOOKUP_ONE(bools);
}
const Point *ParamSet::FindPoint(const string &name, int *nItems) const {
	LOOKUP_PTR(points);
}
Point ParamSet::FindOnePoint(const string &name, const Point &d) const {
	LOOKUP_ONE(points);
}
const Vector *ParamSet::FindVector(const string &name, int *nItems) const {
	LOOKUP_PTR(vectors);
}
Vector ParamSet::FindOneVector(const string &name, const Vector &d) const {
	LOOKUP_ONE(vectors);
}
const Normal *ParamSet::FindNormal(const string &name, int *nItems) const {
	LOOKUP_PTR(normals);
}
Normal ParamSet::FindOneNormal(const string &name, const Normal &d) const {
	LOOKUP_ONE(normals);
}
const RGBColor *ParamSet::FindRGBColor(const string &name, int *nItems) const {
	LOOKUP_PTR(spectra);
}
RGBColor ParamSet::FindOneRGBColor(const string &name, const RGBColor &d) const {
	LOOKUP_ONE(spectra);
}
const string *ParamSet::FindString(const string &name, int *nItems) const {
	LOOKUP_PTR(strings);
}
string ParamSet::FindOneString(const string &name, const string &d) const {
	LOOKUP_ONE(strings);
}
string ParamSet::FindTexture(const string &name) const {
	string d = "";
	LOOKUP_ONE(textures);
}
void ParamSet::ReportUnused() const {
#define CHECK_UNUSED(v) \
	for (i = 0; i < (v).size(); ++i) \
		if (!(v)[i]->lookedUp) \
		{ \
			std::stringstream ss; \
			ss<<"Parameter '"<<(v)[i]->name<<"' not used"; \
			luxError(LUX_NOERROR,LUX_WARNING,ss.str().c_str()); \
		}
	u_int i;
	CHECK_UNUSED(ints);       CHECK_UNUSED(bools);
	CHECK_UNUSED(floats);   CHECK_UNUSED(points);
	CHECK_UNUSED(vectors); CHECK_UNUSED(normals);
	CHECK_UNUSED(spectra); CHECK_UNUSED(strings);
	CHECK_UNUSED(textures);
}
void ParamSet::Clear() {
#define DEL_PARAMS(name) \
	for (u_int i = 0; i < (name).size(); ++i) \
		delete (name)[i]; \
	(name).erase((name).begin(), (name).end())
	DEL_PARAMS(ints);    DEL_PARAMS(bools);
	DEL_PARAMS(floats);  DEL_PARAMS(points);
	DEL_PARAMS(vectors); DEL_PARAMS(normals);
	DEL_PARAMS(spectra); DEL_PARAMS(strings);
	DEL_PARAMS(textures);
#undef DEL_PARAMS
}
string ParamSet::ToString() const {
	string ret;
	u_int i;
	int j;
	string typeString;
	const int bufLen = 48*1024*1024;
	static char *buf = new char[bufLen];
	char *bufEnd = buf + bufLen;
	for (i = 0; i < ints.size(); ++i) {
		char *bufp = buf;
		*bufp = '\0';
		ParamSetItem<int> *item = ints[i];
		typeString = "integer ";
		// Print _ParamSetItem_ declaration, determine how many to print
		int nPrint = item->nItems;
		ret += string("\"");
		ret += typeString;
		ret += item->name;
		ret += string("\"");
		ret += string(" [");
		for (j = 0; j < nPrint; ++j)
			bufp += snprintf(bufp, bufEnd - bufp, "%d ", item->data[j]);
		ret += buf;
		ret += string("] ");
	}
	for (i = 0; i < bools.size(); ++i) {
		char *bufp = buf;
		*bufp = '\0';
		ParamSetItem<bool> *item = bools[i];
		typeString = "bool ";
		// Print _ParamSetItem_ declaration, determine how many to print
		int nPrint = item->nItems;
		ret += string("\"");
		ret += typeString;
		ret += item->name;
		ret += string("\"");
		ret += string(" [");
		for (j = 0; j < nPrint; ++j)
			bufp += snprintf(bufp, bufEnd - bufp, "\"%s\" ", item->data[j] ? "true" : "false");
		ret += buf;
		ret += string("] ");
	}
	for (i = 0; i < floats.size(); ++i) {
		char *bufp = buf;
		*bufp = '\0';
		ParamSetItem<float> *item = floats[i];
		typeString = "float ";
		// Print _ParamSetItem_ declaration, determine how many to print
		int nPrint = item->nItems;
		ret += string("\"");
		ret += typeString;
		ret += item->name;
		ret += string("\"");
		ret += string(" [");
		for (j = 0; j < nPrint; ++j)
			bufp += snprintf(bufp, bufEnd - bufp, "%.8g ", item->data[j]);
		ret += buf;
		ret += string("] ");
	}
	for (i = 0; i < points.size(); ++i) {
		char *bufp = buf;
		*bufp = '\0';
		ParamSetItem<Point> *item = points[i];
		typeString = "point ";
		// Print _ParamSetItem_ declaration, determine how many to print
		int nPrint = item->nItems;
		ret += string("\"");
		ret += typeString;
		ret += item->name;
		ret += string("\"");
		ret += string(" [");
		for (j = 0; j < nPrint; ++j)
			bufp += snprintf(bufp, bufEnd - bufp, "%.8g %.8g %.8g ", item->data[j].x,
				item->data[j].y, item->data[j].z);
		ret += buf;
		ret += string("] ");
	}
	for (i = 0; i < vectors.size(); ++i) {
		char *bufp = buf;
		*bufp = '\0';
		ParamSetItem<Vector> *item = vectors[i];
		typeString = "vector ";
		// Print _ParamSetItem_ declaration, determine how many to print
		int nPrint = item->nItems;
		ret += string("\"");
		ret += typeString;
		ret += item->name;
		ret += string("\"");
		ret += string(" [");
		for (j = 0; j < nPrint; ++j)
			bufp += snprintf(bufp, bufEnd - bufp, "%.8g %.8g %.8g ", item->data[j].x,
				item->data[j].y, item->data[j].z);
		ret += buf;
		ret += string("] ");
	}
	for (i = 0; i < normals.size(); ++i) {
		char *bufp = buf;
		*bufp = '\0';
		ParamSetItem<Normal> *item = normals[i];
		typeString = "normal ";
		// Print _ParamSetItem_ declaration, determine how many to print
		int nPrint = item->nItems;
		ret += string("\"");
		ret += typeString;
		ret += item->name;
		ret += string("\"");
		ret += string(" [");
		for (j = 0; j < nPrint; ++j)
			bufp += snprintf(bufp, bufEnd - bufp, "%.8g %.8g %.8g ", item->data[j].x,
				item->data[j].y, item->data[j].z);
		ret += buf;
		ret += string("] ");
	}
	for (i = 0; i < strings.size(); ++i) {
		char *bufp = buf;
		*bufp = '\0';
		ParamSetItem<string> *item = strings[i];
		typeString = "string ";
		// Print _ParamSetItem_ declaration, determine how many to print
		int nPrint = item->nItems;
		ret += string("\"");
		ret += typeString;
		ret += item->name;
		ret += string("\"");
		ret += string(" [");
		for (j = 0; j < nPrint; ++j)
			bufp += snprintf(bufp, bufEnd - bufp, "\"%s\" ", item->data[j].c_str());
		ret += buf;
		ret += string("] ");
	}
	for (i = 0; i < textures.size(); ++i) {
		char *bufp = buf;
		*bufp = '\0';
		ParamSetItem<string> *item = textures[i];
		typeString = "texture ";
		// Print _ParamSetItem_ declaration, determine how many to print
		int nPrint = item->nItems;
		ret += string("\"");
		ret += typeString;
		ret += item->name;
		ret += string("\"");
		ret += string(" [");
		for (j = 0; j < nPrint; ++j)
			bufp += snprintf(bufp, bufEnd - bufp, "\"%s\" ", item->data[j].c_str());
		ret += buf;
		ret += string("] ");
	}
	for (i = 0; i < spectra.size(); ++i) {
		char *bufp = buf;
		*bufp = '\0';
		ParamSetItem<RGBColor> *item = spectra[i];
		typeString = "color ";
		// Print _ParamSetItem_ declaration, determine how many to print
		int nPrint = item->nItems;
		ret += string("\"");
		ret += typeString;
		ret += item->name;
		ret += string("\"");
		ret += string(" [");
		for (j = 0; j < nPrint; ++j)
			bufp += snprintf(bufp, bufEnd - bufp, "%.8g %.8g %.8g ", item->data[j].c[0],
				item->data[j].c[1], item->data[j].c[2]);
		ret += buf;
		ret += string("] ");
	}
	return ret;
}
// TextureParams Method Definitions
boost::shared_ptr<Texture<SWCSpectrum> >
	TextureParams::GetSWCSpectrumTexture(const string &n,
             const RGBColor &def) const {
	string name = geomParams.FindTexture(n);
	if (name == "") name = materialParams.FindTexture(n);
	if (name != "") {
		if (SWCSpectrumTextures.find(name) != SWCSpectrumTextures.end())
			return SWCSpectrumTextures[name];
		else {
			std::stringstream ss;
			ss << "Couldn't find color texture named '" << name << "' for " << n;
			luxError(LUX_BADTOKEN, LUX_ERROR, ss.str().c_str());
		}
	}
	RGBColor val = geomParams.FindOneRGBColor(n,
		materialParams.FindOneRGBColor(n, def));
	return boost::shared_ptr<Texture<SWCSpectrum> >(new ConstantRGBColorTexture<SWCSpectrum>(val));
}
boost::shared_ptr<Texture<float> > TextureParams::GetFloatTexture(const string &n) const {
	string name = geomParams.FindTexture(n);
	if (name == "") name = materialParams.FindTexture(n);
	if (name != "") {
		if (floatTextures.find(name) != floatTextures.end())
			return floatTextures[name];
		else {
			std::stringstream ss;
			ss << "Couldn't find float texture named '" << name << "' for " << n;
			luxError(LUX_BADTOKEN, LUX_ERROR, ss.str().c_str());
		}
	}
	return boost::shared_ptr<Texture<float> >();
}
boost::shared_ptr<Texture<float> > TextureParams::GetFloatTexture(const string &n,
		float def) const {
	boost::shared_ptr<Texture<float> > texture = GetFloatTexture(n);
	if (texture)  return texture;
	float val = FindFloat(n, def);
	return boost::shared_ptr<Texture<float> >(new ConstantFloatTexture<float>(val));
}

